import numpy as np

"""
Author: Anish Acharya
Contact: anishacharya@utexas.edu
"""


class Compression:
    def __init__(self, num_bits: int,
                 quantization_function: str,
                 dropout_p: float,
                 fraction_coordinates: float):
        self.quantization_function = quantization_function
        self.num_bits = num_bits
        self.fraction_coordinates = fraction_coordinates
        self.dropout_p = dropout_p

    def quantize(self, x):
        if self.quantization_function == 'full':
            return x
        elif self.quantization_function == 'top':
            q = np.zeros_like(x)

            k = round(self.fraction_coordinates * q.shape[0])
            for i in range(0, q.shape[1]):
                indexes = np.argsort(np.abs(x[:, i]))[::-1]
                q[indexes[:k], i] = x[indexes[:k], i]
            return q
        elif self.quantization_function == 'rand':
            q = np.zeros_like(x)
            k = round(self.fraction_coordinates * q.shape[0])
            for i in range(0, q.shape[1]):
                perm_i = np.random.permutation(q.shape[0])
                q[perm_i[0:k], i] = x[perm_i[0:k], i]
            return q
        elif self.quantization_function == 'dropout-biased':
            q = np.zeros_like(x)
            p = self.dropout_p
            for i in range(0, q.shape[1]):
                bin_i = np.random.binomial(1, p, (q.shape[0],))
                q[:, i] = x[:, i] * bin_i
            return q
        elif self.quantization_function == 'dropout-unbiased':
            q = np.zeros_like(x)
            p = self.dropout_p
            for i in range(0, q.shape[1]):
                bin_i = np.random.binomial(1, p, (q.shape[0],))
                q[:, i] = x[:, i] * bin_i
            return q / p
        elif self.quantization_function == 'qsgd':
            q = np.zeros_like(x)
            bits = self.num_bits
            s = 2 ** bits
            tau = 1 + min((np.sqrt(q.shape[0])/s), (q.shape[0]/(s**2)))
            for i in range(0, q.shape[1]):
                unif_i = np.random.rand(q.shape[0],)
                x_i = x[:, i]
                q[:, i] = ((np.sign(x_i) * np.linalg.norm(x_i))/(s*tau)) * np.floor((s*np.abs(x_i)/np.linalg.norm(x_i)) + unif_i)
            return q
        else:
            raise NotImplementedError




import os
import pickle
import glob


def pickle_it(var, name, directory):
    with open(os.path.join(directory, "{}.pickle".format(name)), 'wb') as f:
        pickle.dump(var, f)


def unpickle_dir(d):
    data = {}
    assert os.path.exists(d), "{} does not exists".format(d)
    for file in glob.glob(os.path.join(d, '*.pickle')):
        name = os.path.basename(file)[:-len('.pickle')]
        with open(file, 'rb') as f:
            var = pickle.load(f)
        data[name] = var
    return data



import numpy as np
from scipy.sparse import isspmatrix
from typing import Dict
"""
Author: Anish Acharya
Contact: anishacharya@utexas.edu
"""


class LogisticRegression:
    def __init__(self, params):
        self.params = params

        self.x_estimate = None
        self.Z = None
        self.S = None

    def loss(self, A, y):
        x = np.copy(self.x_estimate)
        x = np.mean(x, axis=1)
        predictions = self.predict(A=A)

        predictions = predictions.reshape(predictions.shape[0], 1)

        # Take the error when label=1
        class1_cost = -y * np.log(predictions)
        # Take the error when label=0
        class2_cost = (1 - y) * np.log(1 - predictions)
        loss = class1_cost - class2_cost
        loss = loss.sum() / A.shape[0]

        # loss = np.sum(np.log(1 + np.exp(-y * (A @ x)))) / A.shape[0]
        if self.params.regularizer:
            loss += self.params.regularizer * np.square(x).sum() / 2
        return loss

    def predict(self, A, machine=None):
        """
          Returns 1D array of probabilities
          that the class label == 1
        """
        if machine is None:
            x = np.copy(self.x_estimate)
            x = np.mean(x, axis=1)
        else:
            x = self.x_estimate[:, machine]
        logits = A @ x
        pred = self.sigmoid(logits)
        return pred
    
    @staticmethod
    def decision_boundary(prob):
        return 1 if prob >= .5 else 0

    @staticmethod
    def sigmoid(z):
        return 1.0 / (1 + np.exp(-z))

    def classify(self, predictions):
        """
        input  - N element array of predictions between 0 and 1
        output - N element array of 0s (False) and 1s (True)
        """
        decision_boundary = np.vectorize(self.decision_boundary)
        return decision_boundary(predictions).flatten()

    @staticmethod
    def accuracy(predicted_labels, actual_labels):
        predicted_labels = predicted_labels.reshape(predicted_labels.shape[0], 1)
        diff = predicted_labels - actual_labels
        return 1.0 - (float(np.count_nonzero(diff)) / len(diff))

    def get_grad(self, machine: int, A, y, stochastic: bool, indices: Dict):
        x = self.x_estimate[:, machine]
        if stochastic:
            # compute stochastic gradient
            sample_idx = np.random.choice(indices[machine])
            sliced_A = A[sample_idx]
            minus_grad = y[sample_idx] * sliced_A * self.sigmoid(-y[sample_idx] * sliced_A.dot(x).squeeze())
            if isspmatrix(sliced_A):
                minus_grad = minus_grad.toarray().squeeze(0)

        else:
            # compute full gradient
            data_ix = indices[machine]
            sliced_A = A[data_ix, :]
            sliced_y = y[data_ix]
            N = sliced_A.shape[0]

            # Get Predictions
            predictions = self.predict(A=sliced_A, machine=machine)
            predictions = predictions.reshape(predictions.shape[0], 1)
            gradient = sliced_A.T @ (predictions - sliced_y)
            gradient /= N
            minus_grad = - gradient

        if self.params.regularizer:
            minus_grad -= self.params.regularizer * (x.reshape(x.shape[0], 1))

        return np.squeeze(minus_grad)

    # def update_estimate(self, t):
    #     t = int(t)  # to avoid overflow with np.int32
    #     p = self.params
    #     if p.estimate == 'final':
    #         self.x_estimate = self.x
    #     elif p.estimate == 'mean':
    #         rho = 1 / (t + 1)
    #         self.x_estimate = self.x_estimate * (1 - rho) + self.x * rho
    #     elif p.estimate == 't+tau':
    #         rho = 2 * (t + p.tau) / ((1 + t) * (t + 2 * p.tau))
    #         self.x_estimate = self.x_estimate * (1 - rho) + self.x * rho
    #     elif p.estimate == '(t+tau)^2':
    #         rho = 6 * ((t + p.tau) ** 2) / ((1 + t) * (6 * (p.tau ** 2) + t + 6 * p.tau * t + 2 * (t ** 2)))
    #         self.x_estimate = self.x_estimate * (1 - rho) + self.x * rho

    def lr(self, epoch, iteration, num_samples):
        if self.params.lr_type == 'constant':
            return self.params.initial_lr
        elif self.params.lr_type == 'epoch_decay':
            return self.params.initial_lr * (self.params.epoch_decay_lr ** epoch)






import torchvision.datasets as datasets
from sklearn.model_selection import train_test_split
from sklearn.datasets import load_breast_cancer
from sklearn.utils import shuffle
import numpy as np
import time
from sklearn import preprocessing

"""
Author: Anish Acharya
Contact: anishacharya@utexas.edu
"""


class DataReader:
    def __init__(self, root: str, data_set: str, download=True, test_split: float = 0.2):
        self.data_set = data_set
        self.root = root
        self.download = download
        if data_set == 'mnist':
            self.A_train, self.y_train, self.A_test, self.y_test = self._get_mnist()
        elif data_set == 'mnist_partial':
            self.A_train, self.y_train, self.A_test, self.y_test = self._get_mnist_partial()
        elif data_set == 'cifar10':
            self.A_train, self.y_train, self.A_test, self.y_test = self._get_cifar10()
        elif data_set == 'breast_cancer':
            self.A_train, self.y_train, self.A_test, self.y_test = self._get_breast_cancer(test_split=test_split)
        else:
            raise NotImplementedError

    def _get_mnist(self, do_sorting=True):
        mnist_train = datasets.MNIST(root=self.root, download=self.download, train=True)
        mnist_test = datasets.MNIST(root=self.root, download=self.download, train=False)

        x_train = mnist_train.train_data.numpy()/255.0
        x_test = mnist_test.test_data.numpy()/255.0
        y_train = mnist_train.train_labels.numpy()
        y_test = mnist_test.test_labels.numpy()

        # flatten the images
        x_train = x_train.reshape(x_train.shape[0], x_train.shape[1]*x_train.shape[2])
        x_test = x_test.reshape(x_test.shape[0], x_test.shape[1]*x_test.shape[2])

        # convert to binary labels
        y_train[y_train < 5] = 0
        y_train[y_train >= 5] = 1
        y_test[y_test < 5] = 0
        y_test[y_test >= 5] = 1

        # # for 4 vs. 9
        # idx_train = np.argwhere(y_train % 5 == 4)
        # idx_test = np.argwhere(y_test % 5 == 4)
        #
        # # for 2 vs. 7
        # # idx_train = np.argwhere(y_train % 5 == 2)
        # # idx_test = np.argwhere(y_test % 5 == 2)
        #
        # x_train = x_train[idx_train[:, 0], :]
        # x_test = x_test[idx_test[:, 0], :]
        #
        # y_train = y_train[idx_train[:, 0]]
        # y_train[y_train == 4] = 0
        # y_train[y_train == 9] = 1
        # y_test = y_test[idx_test[:, 0]]
        # y_test[y_test == 4] = 0
        # y_test[y_test == 9] = 1
        
        if do_sorting:
            y_sorted_ix = np.argsort(y_train)
            x_train = x_train[y_sorted_ix]
            y_train = y_train[y_sorted_ix]

        y_train = y_train.reshape(y_train.shape[0], 1)
        y_test = y_test.reshape(y_test.shape[0], 1)

        # Now add the Bias term - Add a Fake dim of all 1s to the parameters
        x_train_aug = np.ones((x_train.shape[0], x_train.shape[1] + 1))
        x_train_aug[:, 0:x_train.shape[1]] = x_train

        x_test_aug = np.ones((x_test.shape[0], x_test.shape[1] + 1))
        x_test_aug[:, 0:x_test.shape[1]] = x_test

        return x_train_aug, y_train, x_test_aug, y_test

    def _get_mnist_partial(self, do_sorting=True):
        mnist_train = datasets.MNIST(root=self.root, download=self.download, train=True)
        mnist_test = datasets.MNIST(root=self.root, download=self.download, train=False)

        x_train = mnist_train.train_data.numpy() / 255.0
        x_test = mnist_test.test_data.numpy() / 255.0
        y_train = mnist_train.train_labels.numpy()
        y_test = mnist_test.test_labels.numpy()

        # flatten the images
        x_train = x_train.reshape(x_train.shape[0], x_train.shape[1] * x_train.shape[2])
        x_test = x_test.reshape(x_test.shape[0], x_test.shape[1] * x_test.shape[2])

        # Keep only labels 4 and 9
        idx_train = np.argwhere(y_train % 5 == 4)
        idx_test = np.argwhere(y_test % 5 == 4)
        x_train = x_train[idx_train[:, 0], :]
        x_test = x_test[idx_test[:, 0], :]

        y_train = y_train[idx_train[:, 0]]
        y_train[y_train == 4] = 0
        y_train[y_train == 9] = 1

        y_test = y_test[idx_test[:, 0]]
        y_test[y_test == 4] = 0
        y_test[y_test == 9] = 1

        if do_sorting:
            y_sorted_ix = np.argsort(y_train)
            x_train = x_train[y_sorted_ix]
            y_train = y_train[y_sorted_ix]

        y_train = y_train.reshape(y_train.shape[0], 1)
        y_test = y_test.reshape(y_test.shape[0], 1)

        # Now add the Bias term - Add a Fake dim of all 1s to the parameters
        x_train_aug = np.ones((x_train.shape[0], x_train.shape[1] + 1))
        x_train_aug[:, 0:x_train.shape[1]] = x_train

        x_test_aug = np.ones((x_test.shape[0], x_test.shape[1] + 1))
        x_test_aug[:, 0:x_test.shape[1]] = x_test

        return x_train_aug, y_train, x_test_aug, y_test

    def _get_cifar10(self, do_sorting=True):
        cifar10_train = datasets.CIFAR10(root='./data', download=self.download, train=True)
        cifar10_test = datasets.CIFAR10(root='./data', download=self.download, train=False)

        x_train = cifar10_train.data/255.0
        x_test = cifar10_test.data/255.0

        y_train = np.asarray(cifar10_train.targets)
        y_test = np.asarray(cifar10_test.targets)

        y_train = y_train.reshape(y_train.shape[0], 1)
        y_test = y_test.reshape(y_test.shape[0], 1)

        # convert to binary labels
        y_train[y_train < 5] = 0
        y_train[y_train >= 5] = 1
        y_test[y_test < 5] = 0
        y_test[y_test >= 5] = 1

        # Convert to gray-scale
        x_train = (x_train[:, :, :, 0] + x_train[:, :, :, 1] + x_train[:, :, :, 2])/3.0
        x_test = (x_test[:, :, :, 0] + x_test[:, :, :, 1] + x_test[:, :, :, 2])/3.0
        x_train = x_train.reshape(x_train.shape[0], x_train.shape[1]*x_train.shape[2])
        x_test = x_test.reshape(x_test.shape[0], x_test.shape[1]*x_test.shape[2])

        if do_sorting:
            y_sorted_ix = np.argsort(y_train)
            x_train = x_train[y_sorted_ix]
            y_train = y_train[y_sorted_ix]

        # Now add the Bias term - Add a Fake dim of all 1s to the parameters
        x_train_aug = np.ones((x_train.shape[0], x_train.shape[1] + 1))
        x_train_aug[:, 0:x_train.shape[1]] = x_train

        x_test_aug = np.ones((x_test.shape[0], x_test.shape[1] + 1))
        x_test_aug[:, 0:x_test.shape[1]] = x_test

        return x_train_aug, y_train, x_test_aug, y_test
        
    @staticmethod
    def _get_breast_cancer(test_split, do_sorting=True):
        print('Reading Breast Cancer Data')
        t0 = time.time()
        data_bunch = load_breast_cancer()
        x = data_bunch.data
        x = preprocessing.scale(x)
        y = data_bunch.target
        x, y = shuffle(x, y)

        x_train, x_test, y_train, y_test = train_test_split(x, y, test_size=test_split)

        if do_sorting:
            y_sorted_ix = np.argsort(y_train)
            x_train = x_train[y_sorted_ix]
            y_train = y_train[y_sorted_ix]

        y_train = y_train.reshape(y_train.shape[0], 1)
        y_test = y_test.reshape(y_test.shape[0], 1)

        # Now add the Bias term - Add a Fake dim of all 1s to the parameters
        x_train_aug = np.ones((x_train.shape[0], x_train.shape[1] + 1))
        x_train_aug[:, 0:x_train.shape[1]] = x_train

        x_test_aug = np.ones((x_test.shape[0], x_test.shape[1] + 1))
        x_test_aug[:, 0:x_test.shape[1]] = x_test
        print('Time to read Breast Cancer Data = {}s'.format(time.time() - t0))
        return x_train_aug, y_train, x_test_aug, y_test



import time
from dec_opt.data_reader import DataReader
from dec_opt.dec_gd import DecGD


def run_exp(args, model):
    data_set = args.d
    root = args.r
    """ 
    Get Data specifically,  AX = Y , 
    A [samples x Feature] 
    X parameters to estimate 
    Y Target
    """
    print("loading data-set")
    t0 = time.time()
    data_reader = DataReader(root=root, data_set=data_set, download=True)
    print('Time taken to load Data {} sec'.format(time.time() - t0))

    dec_gd = DecGD(data_reader=data_reader,
                   hyper_param=args,
                   model=model)
    return dec_gd.train_losses, dec_gd.test_losses



import numpy as np
import networkx

"""
Author: Anish Acharya
Contact: anishacharya@utexas.edu
"""


class GossipMatrix:
    def __init__(self, topology, n_cores):
        self.nodes = n_cores
        self.topology = topology
        self.W = self._get_gossip_matrix(topology=self.topology, nodes=self.nodes)

    # noinspection PyPep8Naming
    @staticmethod
    def _get_gossip_matrix(topology, nodes):
        if topology == 'ring':
            W = np.zeros(shape=(nodes, nodes))
            value = 1. / 3 if nodes >= 3 else 1. / 2
            np.fill_diagonal(W, value)
            np.fill_diagonal(W[1:], value, wrap=False)
            np.fill_diagonal(W[:, 1:], value, wrap=False)
            W[0, nodes - 1] = value
            W[nodes - 1, 0] = value
            return W
        elif topology == 'centralized':
            W = np.ones((nodes, nodes), dtype=np.float64) / nodes
            return W
        elif topology == 'disconnected':
            W = np.eye(nodes)
            return W
        elif topology == 'torus':
            assert int(np.sqrt(nodes)) ** 2 == nodes
            G = networkx.generators.lattice.grid_2d_graph(int(np.sqrt(nodes)),
                                                          int(np.sqrt(nodes)), periodic=True)
            W = networkx.adjacency_matrix(G).toarray()
            for i in range(0, W.shape[0]):
                W[i][i] = 1
            W = W / 5
            return W
        else:
            raise NotImplementedError


if __name__ == '__main__':
    gossip_matrix_1 = GossipMatrix(topology='ring', n_cores=10).W
    gossip_matrix_2 = GossipMatrix(topology='centralized', n_cores=10).W



import numpy as np
import time
from dec_opt.gossip_matrix import GossipMatrix
from dec_opt.compression import Compression

"""
Author: Anish Acharya
Contact: anishacharya@utexas.edu
"""


class DecGD:
    def __init__(self, data_reader, hyper_param, model):
        self.A_train = data_reader.A_train
        self.y_train = data_reader.y_train
        self.A_test = data_reader.A_test
        self.y_test = data_reader.y_test

        self.param = hyper_param
        self.model = model

        self.W = GossipMatrix(topology=self.param.topology, n_cores=self.param.n_cores).W
        self.C = Compression(num_bits=self.param.num_bits,
                             quantization_function=self.param.quantization_function,
                             dropout_p=self.param.dropout_p,
                             fraction_coordinates=self.param.fraction_coordinates)

        # initialize parameters for each node. Ax = y is the problem we are solving
        # ----------------------------------------------------------------------------------
        self.losses = np.zeros(self.param.epochs + 1)
        self.num_samples, self.num_features = self.A_train.shape
        INIT_WEIGHT_STD = 1 / np.sqrt(self.num_features)

        np.random.seed(self.param.seed)
        self.model.x_estimate = np.random.normal(0, INIT_WEIGHT_STD, size=(self.num_features, self.param.n_cores))
        # self.model.x_estimate = np.tile(self.model.x_estimate, (self.param.n_cores, 1)).T

        self.model.Z = np.zeros(self.model.x_estimate.shape)
        self.model.S = np.zeros(self.model.x_estimate.shape)

        # Now Distribute the Data among machines
        # ----------------------------------------
        self.data_partition_ix, self.num_samples_per_machine = self._distribute_data()

        # Decentralized Training
        # --------------------------
        self.train_losses, self.test_losses = self._dec_train()

    def _distribute_data(self):
        data_partition_ix = []
        num_samples_per_machine = self.num_samples // self.param.n_cores
        all_indexes = np.arange(self.num_samples)
        # np.random.shuffle(all_indexes)

        for machine in range(0, self.param.n_cores - 1):
            data_partition_ix += [
                all_indexes[num_samples_per_machine * machine: num_samples_per_machine * (machine + 1)]]
        # put the rest in the last machine
        data_partition_ix += [all_indexes[num_samples_per_machine * (self.param.n_cores - 1):]]
        print("All but last machine has {} data points".format(num_samples_per_machine))
        print("length of last machine indices:", len(data_partition_ix[-1]))
        return data_partition_ix, num_samples_per_machine

    def _dec_train(self):
        train_losses = np.zeros(self.param.epochs + 1)
        test_losses = np.zeros(self.param.epochs + 1)
        train_losses[0] = self.model.loss(self.A_train, self.y_train)
        test_losses[0] = self.model.loss(self.A_test, self.y_test)
        train_start = time.time()
        for epoch in np.arange(self.param.epochs):
            loss = self.model.loss(self.A_train, self.y_train)
            if np.isinf(loss) or np.isnan(loss):
                print("training exit - diverging")
                break
            lr = self.model.lr(epoch=epoch,
                               iteration=epoch,
                               num_samples=self.num_samples_per_machine)

            # Gradient step
            # --------------------------
            x_plus = np.zeros_like(self.model.x_estimate)
            # for t in 0...T − 1 do in parallel for all workers i ∈[n]
            for machine in range(0, self.param.n_cores):
                # Compute neg. Gradient (or stochastic gradient) based on algorithm
                minus_grad = self.model.get_grad(A=self.A_train,
                                                 y=self.y_train,
                                                 stochastic=self.param.stochastic,
                                                 indices=self.data_partition_ix,
                                                 machine=machine)
                x_plus[:, machine] = lr * minus_grad
            # machine = range(0, self.param.n_cores)
            # pool = Pool(processes=self.param.n_proc)
            # get_grad_multi = partial(self.model.get_grad,
            #                          A=self.A,
            #                          y=self.y,
            #                          stochastic=self.param.stochastic,
            #                          indices=self.data_partition_ix)
            # minus_grad_list = pool.map(get_grad_multi, machine)
            # pool.close()
            # for i in range(0, self.param.n_cores):
            #     x_plus[:, i] = lr * minus_grad_list[i]

            # x_(t+1/2) = x_(t) - lr * grad - Do GD Update
            x_cap = self.model.x_estimate + x_plus

            # Communication step
            # -----------------------------------------
            if self.param.algorithm == 'exact_comm':
                # Xiao, Boyd; Fast Linear Iterations for Distributed Averaging
                self.model.x_estimate = x_cap @ self.W
            elif self.param.algorithm == 'ours':
                # local gradient update i.e. X_t0
                self.model.x_estimate = x_cap
                # now iterate and update the estimate to X_tQ
                for i in range(0, self.param.Q):
                    # Exchanging messages
                    message_exchange = self.C.quantize(self.model.x_estimate - self.model.Z)
                    self.model.S = self.model.S + message_exchange @ self.W
                    # Compression error feedback
                    # error_feedback = self.C.quantize(self.model.x_estimate - self.model.Z)
                    error_feedback = message_exchange
                    self.model.Z = self.model.Z + error_feedback
                    # Local gossip update
                    gossip_update = (self.model.S - self.model.Z)
                    self.model.x_estimate = self.model.x_estimate + \
                        self.param.consensus_lr * gossip_update
            elif self.param.algorithm == 'choco-sgd':
                # Koloskove,Stich,Jaggi; Decentralized Stochastic
                # Optimization and Gossip Algorithms with Compressed Communication
                # x_(t+1) = x_(t+1/2) + \gamma W.dot.(x^_j(t+1) - x^_i(t+1))
                self.model.x_estimate = x_cap + \
                        self.param.consensus_lr * self.model.x_hat.dot(self.W - np.eye(self.param.n_cores))
                pass
            else:
                print('Running Plain GD. n_cores = 1 else convergence results not guaranteed')
                # do nothing just plain GD
                self.model.x_estimate = x_cap

            train_losses[epoch + 1] = self.model.loss(self.A_train, self.y_train)
            test_losses[epoch + 1] = self.model.loss(self.A_test, self.y_test)

            train_acc = compute_accuracy(model=self.model, feature=self.A_train, target=self.y_train)
            test_acc = compute_accuracy(model=self.model, feature=self.A_test, target=self.y_test)
            print("epoch : {}; loss: {}; Train Acc: {}; Test Acc: {}".
                  format(epoch, train_losses[epoch + 1], train_acc, test_acc))
            # print("epoch : {}; loss: {}; Test accuracy : {}".format(epoch, test_losses[epoch + 1], test_acc))
            if np.isinf(train_losses[epoch + 1]) or np.isnan(train_losses[epoch + 1]):
                print("Break training - Diverged")
                break
        print("Training took: {}s".format(time.time() - train_start))
        return train_losses, test_losses


def compute_accuracy(model, feature, target):
    pred = model.predict(A=feature)
    pred_labels = model.classify(predictions=pred)
    acc = model.accuracy(pred_labels, target)
    return acc



import pickle
import argparse
from sklearn.datasets import load_svmlight_file
import time


def _parse_args():
    parser = argparse.ArgumentParser(description='pickle_data.py')
    parser.add_argument('--i', type=str, default='data/rcv1_test.binary.bz2',
                        help='Pass data-set path')
    parser.add_argument('--o', type=str, default='data/rcv1_test.pickle',
                        help='Pass output file path')
    args = parser.parse_args()
    return args


if __name__ == '__main__':
    args = _parse_args()
    print(args)
    input_file = args.i
    output_file = args.o
    t0 = time.time()
    A, y = load_svmlight_file(f=input_file)
    print("Loading SVMLight File took {} sec.".format(time.time()-t0))
    print("The file is huge hence caching into a pickle file for future use")
    A = A.toarray()
    with open(output_file, 'wb') as pickle_file:
        pickle.dump((A, y), pickle_file, protocol=4)
